/*
 wtv020lib v1.0
 by xmaspako, January 7th, 2014
 
 based on Wtv020sd16p.cpp Version 1.1 for Arduino
 http://forum.arduino.cc/index.php?topic=117009.240

 --------------------------------------------------
 
connection pins alias must be defined following the next scheme
(using of 100R resistors on lines is suggested to limit current on PIC pins)

           WTV020_CL = clock pin 7 (P04) on WTV020
           WTV020_DA = data  pin 10 (P05) on WTV020
           WTV020_RS = reset pin 1 (RESET) on WTV020
           WTV020_BS = busy  pin 15 (P06) on WTV020
           WTV020_CL_Direction = clock pin direction
           WTV020_DA_Direction = data pin direction
           WTV020_RS_Direction = reset pin direction
           WTV020_BS_Direction = busy pin direction

Example of pins alias definition:

// WTV020 pinout settings
sbit WTV020_RS at RB0_bit;
sbit WTV020_CL at RB1_bit;
sbit WTV020_DA at RB2_bit;
sbit WTV020_BS at RB3_bit;

// WTV020 pin directions
sbit WTV020_RS_Direction at TRISB0_bit;
sbit WTV020_CL_Direction at TRISB1_bit;
sbit WTV020_DA_Direction at TRISB2_bit;
sbit WTV020_BS_Direction at TRISB3_bit;
 
*/

const unsigned int _PLAY_PAUSE = 0xFFFE;
const unsigned int _STOP       = 0xFFFF;
const unsigned int _VOLUME_MIN = 0xFFF0;
const unsigned int _VOLUME_MAX = 0xFFF7;

void Wtv020_Init(){
  //setting up direction
  WTV020_CL_Direction=0;
  WTV020_DA_Direction=0;
  WTV020_RS_Direction=0;
  WTV020_BS_Direction=1;
  WTV020_CL=0;
  WTV020_RS=1;
  //reset pulse
  WTV020_RS=0;
  delay_ms(100);
  WTV020_RS=1;
  //reset idle to start bit
  WTV020_CL=1;
  delay_ms(300);
}

//send command to module
void Wtv020_SendCommand(unsigned int command){
  int mask;
  WTV020_CL=0;                           //clock low level pulse
  delay_ms(2);                           //initial delay
  for(mask=15; mask>=0; mask--){
    delay_us(50);                        //clock timing
    WTV020_DA=(command>>mask)&0b01 >0;   //transfer command bit to data pin
    delay_us(50);                        //clock timing
    WTV020_CL=1;                         //clock high level pulse
    delay_us(50);                        //clock timing
    WTV020_DA=!(mask>0);                 //set data pin to low (till last bit)
    delay_us(50);                        //clock timing
    WTV020_CL=!(mask>0);                 //clock low level pulse (till last bit)
  }
  delay_ms(20);                          //busy active high from last data bit latch
}

//play entire song and wait till song ends
void Wtv020_Play(unsigned int voiceNumber){
  Wtv020_SendCommand(voiceNumber);
  delay_ms(20);              //wait 20ms for busy to be active
  while(WTV020_BS){}         //do nothing and wait while BUSY is high
}

//start playing song and return
void Wtv020_AsyncPlay(unsigned int voiceNumber){
  Wtv020_SendCommand(voiceNumber);
}

//stop playing song
void Wtv020_Stop(){
  Wtv020_SendCommand(_STOP);
}

//pause/resume song
void Wtv020_Pause(){
  Wtv020_SendCommand(_PLAY_PAUSE);
}

//mute sound
void Wtv020_Mute(){
  Wtv020_SendCommand(_VOLUME_MIN);
}

//set volume (from 0 to 7)
void Wtv020_Volume(unsigned short volume){
  if(volume<8) Wtv020_SendCommand(_VOLUME_MIN+volume);
  else         Wtv020_SendCommand(_VOLUME_MAX);
}